@@ -179,7 +179,7 @@ reset_session (GdmSimpleSlave *slave)
 {
         destroy_session (slave);
         create_new_session (slave);
-        gdm_session_open (GDM_SESSION (slave->priv->session));
+        gdm_session_start_conversation (GDM_SESSION (slave->priv->session));
 }
 
 static gboolean
@@ -418,7 +418,7 @@ static void
 on_session_accredited (GdmSession     *session,
                        GdmSimpleSlave *slave)
 {
-        queue_start_session (slave);
+        gdm_session_open_session (session);
 }
 
 static void
@@ -456,6 +456,27 @@ on_session_accreditation_failed (GdmSession     *session,
         queue_greeter_reset (slave);
 }
 
+static void
+on_session_opened (GdmSession     *session,
+                   GdmSimpleSlave *slave)
+{
+        queue_start_session (slave);
+}
+
+static void
+on_session_open_failed (GdmSession     *session,
+                        const char     *message,
+                        GdmSimpleSlave *slave)
+{
+        if (slave->priv->greeter_server != NULL) {
+                gdm_greeter_server_problem (slave->priv->greeter_server,
+                                            _("Unable to open session"));
+        }
+
+        destroy_session (slave);
+        queue_greeter_reset (slave);
+}
+
 static void
 on_session_info (GdmSession     *session,
                  const char     *text,
@@ -496,15 +517,15 @@ on_session_secret_info_query (GdmSession     *session,
 }
 
 static void
-on_session_opened (GdmSession     *session,
-                   GdmSimpleSlave *slave)
+on_session_conversation_started (GdmSession     *session,
+                                 GdmSimpleSlave *slave)
 {
         gboolean res;
         gboolean enabled;
         char    *username;
         int      delay;
 
-        g_debug ("GdmSimpleSlave: session opened");
+        g_debug ("GdmSimpleSlave: session conversation started");
         if (slave->priv->greeter_server != NULL) {
                 res = gdm_greeter_server_ready (slave->priv->greeter_server);
                 if (! res) {
@@ -615,8 +636,8 @@ create_new_session (GdmSimpleSlave *slave)
         g_free (display_hostname);
 
         g_signal_connect (slave->priv->session,
-                          "opened",
-                          G_CALLBACK (on_session_opened),
+                          "conversation-started",
+                          G_CALLBACK (on_session_conversation_started),
                           slave);
         g_signal_connect (slave->priv->session,
                           "setup-complete",
@@ -658,6 +679,14 @@ create_new_session (GdmSimpleSlave *slave)
                           "accreditation-failed",
                           G_CALLBACK (on_session_accreditation_failed),
                           slave);
+        g_signal_connect (slave->priv->session,
+                          "session-opened",
+                          G_CALLBACK (on_session_opened),
+                          slave);
+        g_signal_connect (slave->priv->session,
+                          "session-open-failed",
+                          G_CALLBACK (on_session_open_failed),
+                          slave);
         g_signal_connect (slave->priv->session,
                           "info",
                           G_CALLBACK (on_session_info),
@@ -833,7 +862,7 @@ on_greeter_connected (GdmGreeterServer *greeter_server,
 
         g_debug ("GdmSimpleSlave: Greeter connected");
 
-        gdm_session_open (GDM_SESSION (slave->priv->session));
+        gdm_session_start_conversation (GDM_SESSION (slave->priv->session));
 
         g_object_get (slave,
                       "display-is-local", &display_is_local,
@@ -1033,6 +1062,8 @@ idle_connect_to_display (GdmSimpleSlave *slave)
                         start_greeter (slave);
                         create_new_session (slave);
                 } else {
+                        /* Run the init script. gdmslave suspends until script has terminated */
+                        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/Init", GDM_USERNAME);
                         reset_session (slave);
                 }
         } else {
